/***********************************************************************

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

/**
 * Title:        <p>
 * Description:  <p>
 * Copyright:    Copyright (c) <p>
 * Company:      <p>
 * @author  Hans Sandstrm
 * @version 1.0
 */

package gecco.game.dke;

import gecco.server.core.*;

public class AttackAction extends Action {

    private int attackFactor;
    private String unitMessage = null;
    private boolean firstRun = true;
    private boolean mainAttack = false;
    
    public AttackAction() {
	super();
    }
    
    public void initiate(int _actionHandle, int _unitHandle) {
	throw new IllegalArgumentException("This action must be instanciated with a point as an argument.");
    }
    
    public void initiate(int _actionHandle, int _unitHandle, int _argUnitHandle) {
	super.initiate(_actionHandle, _unitHandle, _argUnitHandle);

	if (!(getArgumentUnit() instanceof WarUnit)) {
	    unitMessage = Def.cantAttackThisUnitTypeMessage();
	}

	if (getArgumentUnit().isDestroyed()) {
	    unitMessage = Def.getTargetDestroyedMessage();
	    return;
	}
	
	if (((WarUnit)getUnit()).isInCombat()) {
	    unitMessage = Def.getAllreadyInCombatMessage();
	}
	
	double curX = getUnit().getX();
	double curY = getUnit().getY();
	double enemyX = getArgumentUnit().getX();
	double enemyY = getArgumentUnit().getY();
	
	// Check if we're inside the attack radius!
	double dx = enemyX - curX;
	double dy = enemyY - curY;
	double dist = Math.sqrt(dx*dx + dy*dy);
	if (dist > Def.getAttackRadius()) {
	    unitMessage = Def.getAttackOutOfRangeMessage();
	}

	((WarUnit) getUnit()).attackInitiated = true;
	((WarUnit) getArgumentUnit()).underAttackInitiated = true;
    }
    
    public void initiate(int _actionHandle, int _unitHandle, double _argX, double _argY) {
	throw new IllegalArgumentException("This action must be instanciated with a unit as an argument.");
    }
    
    public ActionReturn checkPoint() {
	if (firstRun) {
	    if (unitMessage != null) {
		return new ActionReturnError(unitMessage);
	    }
	    firstRun = false;
	    mainAttack = ((WarUnit)getUnit()).attack((WarUnit)getArgumentUnit());
	    ((WarUnit) getUnit()).attackInitiated = false;
	    ((WarUnit) getArgumentUnit()).underAttackInitiated = false;
	    return new ActionReturnRequeue(Def.getCombatExtendTime());
	    }
	else {
	    if (!mainAttack) {
		return new ActionReturnCompleted();
	    }
	    else {
		((WarUnit)getUnit()).combatEnded();
		return new ActionReturnCompleted();
	    }
	}
    }

    public void prepareAbortion(){
	if (mainAttack) {
	    ((WarUnit)getUnit()).combatEnded();
	} 
    }
	    
}
