/***********************************************************************

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

package gecco.game.dke;

import gecco.server.automaton.*;
import gecco.server.core.UnitEvent;
import java.util.*;
import gecco.server.core.ReferenceHolder;
import java.awt.Color;

public class AutomatonImpl extends Automaton {

    int initialState;
    boolean visited = false;
    int lastVisitor = Def.NO_FORCE;
    public void initialize(int initialState) {
	super.initialize(initialState);
	this.initialState = initialState;
	setColor(initialState);
    }
    
    
    private void setColor(int terrainType) {
	switch(terrainType) {
	case Def.SEA: // Sea
	    setCurrentColor(0,0,192);
	    break;
	case Def.CLEAR: // Clear
	    setCurrentColor(255, 255, 204);
	    break;
	case Def.SWAMP: // Swamp
	    setCurrentColor(0, 128, 128); // Undefined
	    break;
	case Def.RIVER: // River
	    setCurrentColor(64, 64, 255);
	    break;
	case Def.WOODED: // Wooded
	    setCurrentColor(128, 255, 128);
	    break;
	case Def.FOREST: // Forest
	    setCurrentColor(0, 128, 0);
	    break;
	case Def.BEACH: // Beach
	    setCurrentColor(234, 230, 90);
	    break;
	case Def.RAILROAD: // Railroad
	    setCurrentColor(255, 32, 32);
	    break;
	case Def.TOWN: // Town
	    setCurrentColor(192, 192, 192);
	    break;
	case Def.VILLAGE: // Village
	    setCurrentColor(255, 128, 128);
	    break;
	case Def.ROAD_LARGE: // Road large
	    setCurrentColor(0, 0, 0);
	    break;
	case Def.ROAD_SMALL: // Road small
	    setCurrentColor(128, 128, 128);
	    break;
	case Def.BRIDGE: // Bridge
	    setCurrentColor(64, 64, 64);
	    break;
	case Def.UNDER_INDIRECT_FIRE:
	    setCurrentColor(244, 181, 30);
	    break;
	default: // Odefinierat
	    setCurrentColor(200, 0, 0);
	    break;
	}
    }
    
    
    public AutomatonReturn update(int[][] neighourStates) {
	if (getState() == Def.UNDER_INDIRECT_FIRE) {
	    setColor(initialState);
	    setState(initialState);
	}
	return new AutomatonReturnNoAction();
    }
    
    public AutomatonReturn handleEvent(AutomatonEvent event) {
	if (event.getEventName().equals(Def.getIndirectFireString())) {
	    setColor(Def.UNDER_INDIRECT_FIRE);
	    setState(Def.UNDER_INDIRECT_FIRE);
	    return new AutomatonReturnQueueSelf(Def.getTimeToShowIndirectFire(),
						new UnitEvent(Def.getIndirectFireString(), event.getFactor()));
	}
	return new AutomatonReturnNoAction();
    }
    
    public UnitEvent getUnitEventForCurrentState(String unitType) {
	return null;
    }

    public boolean isRoute(){
	if ((initialState & 64) == 64 ) {
	    return true;
	}
	else {
	    return false;   
	}
    }
    
    public boolean isVisited(){
	return visited;
    }
    
    public void setVisited(boolean b){
	visited = b;
    }

    public int getInitialState(){
	return initialState;
    }


    /**
     * Gets the value of lastVisitor
     *
     * @return the value of lastVisitor
     */
    public int getLastVisitor() {
	return this.lastVisitor;
    }

    public void setLastVisitor(int argLastVisitor, int argX, int argY){
	if (argLastVisitor != getLastVisitor()) {
	    setLastVisitor(argLastVisitor);
	    ((DKEReferee) ReferenceHolder.getReferee()).lastVisitorChanged(argLastVisitor);
	    UnitEvent ue = new UnitEvent(Def.getLastVisitorChangedString(), argLastVisitor);
	    ReferenceHolder.getActionProcessor().handleAutomatonUnitEvent(argX, argY, ue);
	}
    }

    /**
     * Sets the value of lastVisitor
     *
     * @param argLastVisitor Value to assign to this.lastVisitor
     */
    public void setLastVisitor(int argLastVisitor){
	this.lastVisitor = argLastVisitor;
     }

}


