/***********************************************************************

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/
package gecco.game.dke;

/**
 * IndirectFireAction.java
 *
 *
 * Created: Wed Aug 28 10:18:05 2002
 *
 * @author <a href="mailto:Hans.Sandstrom@fhs.mil.se">Hans Sandstrm</a>
 * @version
 */

import gecco.server.core.*;
import gecco.server.automaton.AutomatonMap;

public class IndirectFireAction extends Action {

    WarUnit warUnit;
    int indirectFireFactor;
    double actionExtendTime;
    int dispersion;
    int centerX;
    int centerY;
    AutomatonMap map = ReferenceHolder.getAutomatonMap();
    boolean firstRun = true;
    String unitMessage = null;

    public IndirectFireAction (){
	super();
    }
    
    // Override superclass methods
    public void initiate(int _actionHandle, int _unitHandle) {
	throw new IllegalArgumentException("This action must be instanciated with a point as an argument.");
    }

    public void initiate(int _actionHandle, int _unitHandle, int _argUnitHandle) {
	throw new IllegalArgumentException("This action must be instanciated with a point as an argument.");
    }
    
    public void initiate(int _actionHandle, int _unitHandle, double _argX, double _argY) {
	super.initiate(_actionHandle, _unitHandle, _argX, _argY);
	try {
	    warUnit = (WarUnit) getUnit();
	} catch (ClassCastException e) {
	    ReferenceHolder.err.println(e.getCause());
	    ReferenceHolder.err.println(e.getMessage());
	    e.printStackTrace(ReferenceHolder.err);
	    System.exit(1);
	}

	// Check distance
	double curX = getUnit().getX();
	double curY = getUnit().getY();
	double dx = getArgumentX() - curX;
	double dy = getArgumentY() - curY;
	double dist = Math.sqrt(dx*dx + dy*dy);
	if (!Def.isWhithinIndirectFireRange(dist)) {
	    unitMessage = Def.getIndirectFireOutOfRangeMessage();
	}
 	
	centerX = (int)getArgumentX();
	centerY = (int)getArgumentY();
	indirectFireFactor = warUnit.getIndirectFireFactor();
	actionExtendTime = warUnit.getIndirectFireExtendTime();
	dispersion = warUnit.getIndirectFireDispersion();
    }
    
    public ActionReturn checkPoint(){
	if (firstRun) {
	    if (unitMessage != null) {
		return new ActionReturnError(unitMessage);
	    }
	    else {
		if (warUnit.isFiringIndirect() || warUnit.isReloading()) {
		    return new ActionReturnRequeue(5);
		}
		warUnit.setFiringIndirect(true);
		warUnit.updateStatus();
		firstRun = false;
		for (int y = centerY - dispersion; y <= centerY + dispersion; y++) {
		    for (int x = centerX - dispersion; x <= centerX + dispersion ; x++) {
			if (map.areValidCoordinates(x,y)) {
			    sendEventToAutomaton(x, y, Def.getIndirectFireString(), indirectFireFactor);		    
			}
		    }
		}
		new LockDuringIndirectFireDelayAction(getUnitHandle());
		return new ActionReturnRequeue(actionExtendTime, "Startar indirekt beskjutning.");
	    }
	}
	else {
	    return new ActionReturnCompleted("Indirekt beskjutning slutfrd."); 
	}
    }

    public void prepareAbortion(){
    }
	
}
