/***********************************************************************

This program is free software; you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

This program is distributed in the hope that it will be useful, but
WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307,
USA.

***********************************************************************/

/**
 * Title:        <p>
 * Description:  <p>
 * Copyright:    Copyright (c) <p>
 * Company:      <p>
 * @author <a href="mailto:Hans.Sandstrom@fhs.mil.se">Hans Sandstrm</a>
 * @version 1.0
 */

package gecco.game.dke;

import gecco.server.core.*;
import java.util.Vector;

public class MoveAction extends Action {

    double addX;
    double addY;
    WarUnit unit = null;
    boolean firstRun = true;
    String message = null;
    boolean updateVisited = false;

    public void initiate(int _actionHandle, int _unitHandle, double _argX, double _argY) {
      super.initiate(_actionHandle, _unitHandle, _argX, _argY);
      if (isOutsideMap(_argX, _argY)) {
	  message = "Invalid coordinates.";
	  return;
      }
      calcAddXAddY(_argX, _argY);
    }
    
    public void calcAddXAddY(double _argX, double _argY) {
	calcAddXAddY(_argX, _argY, Def.STEP_LENGTH);
    }
    
    public void calcAddXAddY(double _argX, double _argY, double stepLength) {
      // Calculate addX and addY
      double curX = getWarUnit().getX();
      double curY = getWarUnit().getY();
      double dx = _argX - curX;
      double dy = _argY - curY;
      double dist = Math.sqrt(dx*dx + dy*dy);
      double coeff = dist / stepLength;
      addX = dx / coeff;
      addY = dy / coeff;
    }

    public boolean isCloseEnough(double destX, double destY){
	double curX = getWarUnit().getX();
	double curY = getWarUnit().getY();
	if (Math.abs(curX - destX)<Def.STEP_LENGTH && Math.abs(curY - destY) < Def.STEP_LENGTH) {
	    // We're done, set coordinates to destination coordinates and return OK!
	    return true;
	} else return false;
    }

    public WarUnit getWarUnit(){
	return (WarUnit)super.getUnit();
    }

    public AutomatonImpl getAutomatonImpl(int x, int y){
	return (AutomatonImpl)ReferenceHolder.getAutomatonMap().getAutomaton(x,y);
    }

    public boolean updateUpdateVisited(){
	// Check if we should update visited in automaton
	double curX = getWarUnit().getX();
	double curY = getWarUnit().getY();
	updateVisited = false;
	if (getAutomatonImpl((int)(curX+addX), (int)(curY+addY)).isRoute()){
	    if ((int)curX != (int)(curX+addX) || (int)curY != (int)(curY+addY)){
		updateVisited = true;
	    }
	}
	return updateVisited;
    }

    public void testUpdateVisited(){
	if (updateVisited) {
	    int curX = getWarUnit().getIntX();
	    int curY = getWarUnit().getIntY();
	    markAsVisited(curX, curY);
	    expandVisitedToRouteWidth(curX, curY, (Math.abs(addX) > Math.abs(addY)? 1:0));
	}
    }

    public void markAsVisited(int x, int y){
	AutomatonImpl ai = null;
	if (ReferenceHolder.getAutomatonMap().areValidCoordinates(x, y)){
	    ai =(AutomatonImpl)ReferenceHolder.getAutomatonMap().getAutomaton(x, y);
	    ai.setLastVisitor(getWarUnit().getForce(), x, y); // giving the automaton its coordinates
	}
    }

    /**
     * If unit moves along a road i x-directon, expand visited automatons i y-directon
     *	 and vice versa. Max expantion is set to Def.getMaxRoadWidth() in both directions.
     *
     * @param x an <code>int</code> value
     * @param y an <code>int</code> value
     * @param direction an <code>int</code>, 0 expand in X direction anyting else
     * expand in Y direction
     */
    public void expandVisitedToRouteWidth(int x, int y, int direction){
	int i;
	boolean cont = true;
	AutomatonImpl ai = null;
	int maxExpansion = Def.getMaxRoadWidth();
	    
	i = 1;
	if (cont = ReferenceHolder.getAutomatonMap().areValidCoordinates(x, y+i)){
	    ai = (AutomatonImpl)ReferenceHolder.getAutomatonMap().getAutomaton(x, y+i);
	}
	while (cont && ai.isRoute() && i < maxExpansion) {
	    ai.setLastVisitor(getWarUnit().getForce(),x , y+i);
	    i++;
	    if (cont = ReferenceHolder.getAutomatonMap().areValidCoordinates(x, y+i)){
		ai = (AutomatonImpl)ReferenceHolder.getAutomatonMap().getAutomaton(x, y+i);
	    }
	}
	i = -1;
	if (cont = ReferenceHolder.getAutomatonMap().areValidCoordinates(x, y+i)){
	    ai = (AutomatonImpl)ReferenceHolder.getAutomatonMap().getAutomaton(x, y+i);
	}
	while (cont && ai.isRoute() && i > -maxExpansion) {
	    ai.setLastVisitor(getWarUnit().getForce(), x, y+i);
	    i--;
	    if (cont = ReferenceHolder.getAutomatonMap().areValidCoordinates(x, y+i)){
		ai = (AutomatonImpl)ReferenceHolder.getAutomatonMap().getAutomaton(x, y+i);
	    }
	}
	i = 1;
	if (cont = ReferenceHolder.getAutomatonMap().areValidCoordinates(x+i, y)){
	    ai = (AutomatonImpl)ReferenceHolder.getAutomatonMap().getAutomaton(x+i, y);
	}
	while (cont && ai.isRoute() && i < maxExpansion) {
	    ai.setLastVisitor(getWarUnit().getForce(), x+i, y);
	    i++;
	    if (cont = ReferenceHolder.getAutomatonMap().areValidCoordinates(x+i, y)){
		ai = (AutomatonImpl)ReferenceHolder.getAutomatonMap().getAutomaton(x+i, y);
	    }
	}
	i = -1;
	if (cont = ReferenceHolder.getAutomatonMap().areValidCoordinates(x+i, y)){
	    ai = (AutomatonImpl)ReferenceHolder.getAutomatonMap().getAutomaton(x+i, y);
	}
	while (cont && ai.isRoute() && i > -maxExpansion) {
	    ai.setLastVisitor(getWarUnit().getForce(), x+i, y);
	    i--;
	    if (cont = ReferenceHolder.getAutomatonMap().areValidCoordinates(x+i, y)){
		ai = (AutomatonImpl)ReferenceHolder.getAutomatonMap().getAutomaton(x+i, y);
	    }
	}
	
    }

    public ActionReturn checkPoint() {
	if (getWarUnit().isInCombat() || getWarUnit().underAttackInitiated 
	    || getWarUnit().attackInitiated || getWarUnit().isFiringIndirect()) {
	    return new ActionReturnRequeue(5.0);
	}
	
	if (firstRun) {
	    if (message != null) {
		return new ActionReturnError(message);
	    }
	    else {
		int x = getWarUnit().getIntX();
		int y = getWarUnit().getIntY();
		if (((AutomatonImpl)ReferenceHolder.getAutomatonMap().getAutomaton(x,y)).isRoute()) {
		    markAsVisited(x, y);
		    expandVisitedToRouteWidth(x,y,(Math.abs(addX) > Math.abs(addY)? 1:0));
		}

		updateUnitStatus(true);
		firstRun = false;
	    }
	}
	
	// getArgumentX(),getArgumentY() are the destination coordinates
	double curX = getWarUnit().getX();
	double curY = getWarUnit().getY();
	double destX = getArgumentX();
	double destY = getArgumentY();
	calcAddXAddY(destX, destY);
	
	if (getWarUnit().cantMoveTo(getAutomatonState((int)(curX+addX), (int)(curY+addY)))) {
	    updateUnitStatus(false);
	    return new ActionReturnError(Def.getCantMoveToWaterString());
	}
	
	// Check if we're close enough to the endpoint!
	if (isCloseEnough(destX, destY)) {
	    // We're done, set coordinates to destination coordinates and return OK!
	    System.out.println(Def.getDestinationReachedString());
	    updateUpdateVisited();
	    // getWarUnit().setPosition(destX, destY);
	    getWarUnit().setPosition(destX, destY, true);
	    testUpdateVisited();
	    updateUnitStatus(false);
	    return new ActionReturnCompleted();
	}
	
	// Would this movement take us outside the map?
	if (isOutsideMap(curX + addX, curY + addY)) {
	    updateUnitStatus(false);
	    return new ActionReturnError(Def.getCantMoveOutOfMapString());
	}

	updateUpdateVisited();
	
	// Step forward
	curX = curX + addX;
	curY = curY + addY;

	getWarUnit().setPosition(curX, curY, true);
	
	testUpdateVisited();
	
	return new ActionReturnRequeue(getWarUnit().getTimeToRequeueForMovement());
    }

    public void updateUnitStatus(boolean b){
	getWarUnit().setAdvancing(b);
	getWarUnit().updateStatus();
    }

    public void prepareAbortion(){
	updateUnitStatus(false);
    }


}


